@@ -3,13 +3,16 @@ __all__ = [
     "LLMModel",
     "LiteLLMModel",
     "PassThroughRouter",
+    "extract_top_logprobs",
     "rate_limited",
+    "request_limited",
     "sum_logprobs",
     "validate_json_completion",
 ]
 
 import asyncio
 import contextlib
+import copy
 import functools
 import json
 import logging
@@ -21,6 +24,7 @@ from collections.abc import (
     Coroutine,
     Iterable,
     Mapping,
+    Sequence,
 )
 from enum import StrEnum
 from inspect import isasyncgenfunction, isawaitable, signature
@@ -35,6 +39,7 @@ from aviary.core import (
     ToolSelector,
     is_coroutine_callable,
 )
+from aviary.message import MalformedMessageError
 from pydantic import (
     BaseModel,
     ConfigDict,
@@ -72,17 +77,20 @@ class CommonLLMNames(StrEnum):
     """When you don't want to think about models, just use one from here."""
 
     # Use these to avoid thinking about exact versions
+    GPT_5 = "gpt-5-2025-08-07"
+    GPT_5_MINI = "gpt-5-mini-2025-08-07"
     GPT_4O = "gpt-4o-2024-11-20"
     GPT_35_TURBO = "gpt-3.5-turbo-0125"
     CLAUDE_35_SONNET = "claude-3-5-sonnet-20241022"
+    CLAUDE_37_SONNET = "claude-3-7-sonnet-20250219"
 
     # Use these when trying to think of a somewhat opinionated default
     OPENAI_BASELINE = "gpt-4o-2024-11-20"  # Fast and decent
 
     # Use these in unit testing
     OPENAI_TEST = "gpt-4o-mini-2024-07-18"  # Cheap, fast, and not OpenAI's cutting edge
-    ANTHROPIC_TEST = (
-        "claude-3-5-haiku-20241022"  # Cheap, fast, and not Anthropic's cutting edge
+    ANTHROPIC_TEST = (  # Cheap, fast, and not Anthropic's cutting edge
+        "claude-3-5-haiku-20241022"
     )
 
 
@@ -112,6 +120,24 @@ def sum_logprobs(choice: litellm.utils.Choices | list[float]) -> float | None:
     return None
 
 
+def extract_top_logprobs(
+    completion: litellm.utils.Choices,
+) -> list[list[tuple[str, float]]] | None:
+    """Extract the top logprobs from an litellm completion."""
+    logprobs_obj = getattr(completion, "logprobs", None)
+    if logprobs_obj is None:
+        return None
+
+    content = getattr(logprobs_obj, "content", None)
+    if not content or not isinstance(content, list):
+        return None
+
+    return [
+        [(t.token, float(t.logprob)) for t in (getattr(pos, "top_logprobs", []) or [])]
+        for pos in content
+    ]
+
+
 def validate_json_completion(
     completion: litellm.ModelResponse,
     output_type: type[BaseModel] | TypeAdapter | JSONSchema,
@@ -148,8 +174,10 @@ def validate_json_completion(
 
 
 def prepare_args(
-    func: Callable, completion: str, name: str | None
-) -> tuple[tuple, dict]:
+    func: Callable[..., Any] | Callable[..., Awaitable],
+    completion: str,
+    name: str | None,
+) -> tuple[tuple[str, ...], dict[str, Any]]:
     with contextlib.suppress(TypeError):
         if "name" in signature(func).parameters:
             return (completion,), {"name": name}
@@ -221,30 +249,42 @@ class LLMModel(ABC, BaseModel):
     async def call(  # noqa: C901, PLR0915
         self,
         messages: list[Message],
-        callbacks: Iterable[Callable] | None = None,
+        callbacks: (
+            Sequence[Callable[..., Any] | Callable[..., Awaitable]] | None
+        ) = None,
         name: str | None = None,
         output_type: type[BaseModel] | TypeAdapter | JSONSchema | None = None,
         tools: list[Tool] | None = None,
         tool_choice: Tool | str | None = TOOL_CHOICE_REQUIRED,
-        **chat_kwargs,
+        **kwargs,
     ) -> list[LLMResult]:
         """Call the LLM model with the given messages and configuration.
 
-        messages: A list of messages to send to the language model.
-        callbacks: A list of callback functions to execute
-        name: Optional name for the result.
-        output_type: The type of the output.
-        tools: A list of tools to use.
-        tool_choice: The tool choice to use.
+        Args:
+            messages: A list of messages to send to the language model.
+            callbacks: A list of callback functions to execute.
+            name: Optional name for the result.
+            output_type: The type of the output.
+            tools: A list of tools to use.
+            tool_choice: The tool choice to use.
+            kwargs: Additional keyword arguments for the chat completion.
 
-        Results: A list of LLMResult objects containing the result of the call.
+        Returns:
+            A list of LLMResult objects containing the result of the call.
 
         Raises:
             ValueError: If the LLM type is unknown.
         """
+        chat_kwargs = copy.deepcopy(kwargs)
+        # if using the config for an LLMModel,
+        # there may be a nested 'config' key
+        # that can't be used by chat
+        chat_kwargs.pop("config", None)
         n = chat_kwargs.get("n") or self.config.get("n", 1)
         if n < 1:
             raise ValueError("Number of completions (n) must be >= 1.")
+        if "fallbacks" not in chat_kwargs and "fallbacks" in self.config:
+            chat_kwargs["fallbacks"] = self.config.get("fallbacks", [])
 
         # deal with tools
         if tools:
@@ -342,7 +382,7 @@ class LLMModel(ABC, BaseModel):
         for result in results:
             usage = result.prompt_count, result.completion_count
             if not sum(usage):
-                result.completion_count = self.count_tokens(cast(str, result.text))
+                result.completion_count = self.count_tokens(cast("str", result.text))
             result.seconds_to_last_token = (
                 asyncio.get_running_loop().time() - start_clock
             )
@@ -355,19 +395,32 @@ class LLMModel(ABC, BaseModel):
 
     async def call_single(
         self,
-        messages: list[Message],
-        callbacks: Iterable[Callable] | None = None,
+        messages: list[Message] | str,
+        callbacks: (
+            Sequence[Callable[..., Any] | Callable[..., Awaitable]] | None
+        ) = None,
         name: str | None = None,
         output_type: type[BaseModel] | TypeAdapter | JSONSchema | None = None,
         tools: list[Tool] | None = None,
         tool_choice: Tool | str | None = TOOL_CHOICE_REQUIRED,
         **kwargs,
     ) -> LLMResult:
+        if isinstance(messages, str):
+            # convenience for single message
+            messages = [Message(content=messages)]
         results = await self.call(
-            messages, callbacks, name, output_type, tools, tool_choice, n=1, **kwargs
+            messages,
+            callbacks,
+            name,
+            output_type,
+            tools,
+            tool_choice,
+            n=1,
+            **kwargs,
         )
-        if not results:
-            raise ValueError("No results returned from call")
+        if len(results) != 1:
+            # Can be caused by issues like https://github.com/BerriAI/litellm/issues/12298
+            raise ValueError(f"Got {len(results)} results when expecting just one.")
         return results[0]
 
 
@@ -430,6 +483,48 @@ def rate_limited(func):
     return wrapper
 
 
+@overload
+def request_limited(
+    func: Callable[P, Coroutine[Any, Any, list[LLMResult]]],
+) -> Callable[P, Coroutine[Any, Any, list[LLMResult]]]: ...
+
+
+@overload
+def request_limited(
+    func: Callable[P, Coroutine[Any, Any, AsyncIterable[LLMResult]]],
+) -> Callable[P, Coroutine[Any, Any, AsyncIterable[LLMResult]]]: ...
+
+
+def request_limited(func):
+    """Decorator to limit requests per minute for LLMModel methods."""
+
+    @functools.wraps(func)
+    async def wrapper(self, *args, **kwargs):
+        if not hasattr(self, "check_request_limit"):
+            raise NotImplementedError(
+                f"Model {self.name} must have a `check_request_limit` method."
+            )
+
+        await self.check_request_limit()
+
+        if isasyncgenfunction(func):
+
+            async def request_limited_generator() -> AsyncIterable[LLMResult]:
+                first_item = True
+                async for item in func(self, *args, **kwargs):
+                    # Skip rate limit check for first item since we already checked at generator start
+                    if not first_item:
+                        await self.check_request_limit()
+                    else:
+                        first_item = False
+                    yield item
+
+            return request_limited_generator()
+        return await func(self, *args, **kwargs)
+
+    return wrapper
+
+
 class PassThroughRouter(litellm.Router):  # TODO: add rate_limited
     """Router that is just a wrapper on LiteLLM's normal free functions."""
 
@@ -442,6 +537,9 @@ class PassThroughRouter(litellm.Router):  # TODO: add rate_limited
     async def acompletion(self, *args, **kwargs):
         return await litellm.acompletion(*args, **(self._default_kwargs | kwargs))
 
+    async def aembedding(self, *args, **kwargs):
+        return await litellm.aembedding(*args, **(self._default_kwargs | kwargs))
+
 
 class LiteLLMModel(LLMModel):
     """A wrapper around the litellm library."""
@@ -462,39 +560,72 @@ class LiteLLMModel(LLMModel):
             " regardless of `pass_through_router` being present. The optional"
             " `rate_limit` key is a dictionary keyed by model group name with values"
             " of type limits.RateLimitItem (in tokens / minute) or valid"
-            " limits.RateLimitItem string for parsing."
+            " limits.RateLimitItem string for parsing. The optional `request_limit`"
+            " key is a dictionary keyed by model group name with values representing"
+            " the maximum number of requests per minute."
         ),
     )
     _router: litellm.Router | None = None
 
     @model_validator(mode="before")
     @classmethod
-    def maybe_set_config_attribute(cls, data: dict[str, Any]) -> dict[str, Any]:
+    def maybe_set_config_attribute(cls, input_data: dict[str, Any]) -> dict[str, Any]:
         """
         Set the config attribute if it is not provided.
 
         If name is not provided, uses the default name.
         If a user only gives a name, make a sensible config dict for them.
         """
+        data = copy.deepcopy(input_data)
+
+        # unnest the config key if it's nested
+        if "config" in data and "config" in data["config"]:
+            data["config"].update(data["config"]["config"])
+            data["config"].pop("config")
+
         if "config" not in data:
             data["config"] = {}
         if "name" not in data:
             data["name"] = data["config"].get("name", cls.model_fields["name"].default)
         if "model_list" not in data["config"]:
+            is_openai_model = "openai" in litellm.get_llm_provider(data["name"])
+            max_tokens = data["config"].get("max_tokens")
+            if (
+                "logprobs" in data["config"] or "top_logprobs" in data["config"]
+            ) and not is_openai_model:
+                logger.warning(
+                    "Ignoring token logprobs for non-OpenAI model %s, as they are not supported.",
+                    data["name"],
+                )
             data["config"] = {
                 "model_list": [
                     {
                         "model_name": data["name"],
-                        "litellm_params": {
-                            "model": data["name"],
-                            "n": data["config"].get("n", 1),
-                            "temperature": data["config"].get("temperature", 0.1),
-                            "max_tokens": data["config"].get("max_tokens", 4096),
-                        }
-                        | (
-                            {}
-                            if "gemini" not in data["name"]
-                            else {"safety_settings": DEFAULT_VERTEX_SAFETY_SETTINGS}
+                        "litellm_params": (
+                            {
+                                "model": data["name"],
+                                "n": data["config"].get("n", 1),
+                                "temperature": data["config"].get("temperature", 1.0),
+                                "max_tokens": data["config"].get("max_tokens", 4096),
+                            }
+                            | (
+                                {}
+                                if "gemini" not in data["name"]
+                                else {"safety_settings": DEFAULT_VERTEX_SAFETY_SETTINGS}
+                            )
+                            | ({} if max_tokens else {"max_tokens": max_tokens})
+                            | (
+                                {}
+                                if "logprobs" not in data["config"]
+                                or not is_openai_model
+                                else {"logprobs": data["config"]["logprobs"]}
+                            )
+                            | (
+                                {}
+                                if "top_logprobs" not in data["config"]
+                                or not is_openai_model
+                                else {"top_logprobs": data["config"]["top_logprobs"]}
+                            )
                         ),
                     }
                 ],
@@ -519,8 +650,6 @@ class LiteLLMModel(LLMModel):
         else:
             # pylint: disable-next=possibly-used-before-assignment
             _DeploymentTypedDictValidator.validate_python(model_list)
-        if len({m["model_name"] for m in model_list}) > 1:
-            raise ValueError("Only one model name per model list is supported for now.")
         return data
 
     # SEE: https://platform.openai.com/docs/api-reference/chat/create#chat-create-tool_choice
@@ -552,6 +681,16 @@ class LiteLLMModel(LLMModel):
                 )
         return self._router
 
+    async def check_request_limit(self, **kwargs) -> None:
+        """Check if the request is within the request rate limit."""
+        if "request_limit" in self.config:
+            await GLOBAL_LIMITER.try_acquire(
+                ("client|request", self.name),
+                self.config["request_limit"].get(self.name, None),
+                weight=1,
+                **kwargs,
+            )
+
     async def check_rate_limit(self, token_count: float, **kwargs) -> None:
         if "rate_limit" in self.config:
             await GLOBAL_LIMITER.try_acquire(
@@ -561,67 +700,76 @@ class LiteLLMModel(LLMModel):
                 **kwargs,
             )
 
+    # the order should be first request and then rate(token)
+    @request_limited
     @rate_limited
     async def acompletion(self, messages: list[Message], **kwargs) -> list[LLMResult]:
         tools = kwargs.get("tools")
         if not tools:
-            # OpenAI, Anthropic and pottentially other LLM providers
+            # OpenAI, Anthropic and potentially other LLM providers
             # don't allow empty tool_calls lists, so remove empty
             kwargs.pop("tools", None)
 
         # cast is necessary for LiteLLM typing bug: https://github.com/BerriAI/litellm/issues/7641
         prompts = cast(
-            list[litellm.types.llms.openai.AllMessageValues],
+            "list[litellm.types.llms.openai.AllMessageValues]",
             [m.model_dump(by_alias=True) for m in messages],
         )
         completions = await track_costs(self.router.acompletion)(
             self.name, prompts, **kwargs
         )
+        used_model = completions.model or self.name
         results: list[LLMResult] = []
 
         # We are not streaming here, so we can cast to list[litellm.utils.Choices]
-        choices = cast(list[litellm.utils.Choices], completions.choices)
-        for completion in choices:
+        choices = cast("list[litellm.utils.Choices]", completions.choices)
+        for choice in choices:
+            msg_type = (
+                ToolRequestMessage
+                if choice.finish_reason == "tool_calls"
+                or getattr(choice.message, "tool_calls", None) is not None
+                else Message
+            )
+            serialized_message = choice.message.model_dump()
             if (
-                tools is not None  # Allows for empty tools list
-                or completion.finish_reason == "tool_calls"
-                or (getattr(completion.message, "tool_calls", None) is not None)
+                # Confirm we explicitly received an empty tool list, so we don't unnecessarily
+                # make a tool request message over a normal message
+                tools is not None
+                and not tools  # Confirm it's the empty tools special case
+                and not serialized_message.get("tool_calls")  # Don't clobber anything
             ):
-                serialized_message = completion.message.model_dump()
-                serialized_message["tool_calls"] = (
-                    serialized_message.get("tool_calls") or []
-                )
-                output_messages: list[Message | ToolRequestMessage] = [
-                    ToolRequestMessage(**serialized_message)
-                ]
-            else:
-                output_messages = [Message(**completion.message.model_dump())]
+                # This is a design decision made to simplify
+                # downstream language agent logic, where:
+                # 1. We wanted the presence of tools, even if the list is empty,
+                #    to lead to a ToolRequestMessage
+                # 2. However, OpenAI gpt-4o returns null tool_calls if tools is empty,
+                #    not empty tool_calls, which leads to a plain Message
+                # 3. So, we add this special case to make a ToolRequestMessage
+                serialized_message["tool_calls"] = []
+                msg_type = ToolRequestMessage
+            try:
+                output_messages = [msg_type(**serialized_message)]
+            except ValidationError as exc:
+                raise MalformedMessageError(
+                    f"Failed to convert model response's message {choice.message}"
+                    f" into a {msg_type.__name__}."
+                    f" Got finish reason {choice.finish_reason!r},"
+                    f" full response was {completions},"
+                    f" and tool choice was {kwargs.get('tool_choice')!r}."
+                ) from exc
 
             reasoning_content = None
-            if (
-                hasattr(completion.message, "provider_specific_fields")
-                and completion.message.provider_specific_fields is not None
-            ):
-                # NOTE: DeepSeek's reasoning was added in:
-                # https://github.com/BerriAI/litellm/issues/7877
-                provider_specific_fields = completion.message.provider_specific_fields
-                if isinstance(provider_specific_fields, dict):
-                    reasoning_content = provider_specific_fields.get(
-                        "reasoning_content"
-                    )
-            elif hasattr(completion.message, "reasoning"):
-                # OpenRouter's reasoning:
-                # https://github.com/BerriAI/litellm/issues/8130
-                # https://openrouter.ai/docs/api-reference/parameters#include-reasoning
-                reasoning_content = completion.message.reasoning
+            if hasattr(choice.message, "reasoning_content"):
+                reasoning_content = choice.message.reasoning_content
 
             results.append(
                 LLMResult(
-                    model=self.name,
-                    text=completion.message.content,
+                    model=used_model,
+                    text=choice.message.content,
                     prompt=messages,
                     messages=output_messages,
-                    logprob=sum_logprobs(completion),
+                    logprob=sum_logprobs(choice),
+                    top_logprobs=extract_top_logprobs(choice),
                     prompt_count=completions.usage.prompt_tokens,  # type: ignore[attr-defined]
                     completion_count=completions.usage.completion_tokens,  # type: ignore[attr-defined]
                     system_fingerprint=completions.system_fingerprint,
@@ -630,24 +778,21 @@ class LiteLLMModel(LLMModel):
             )
         return results
 
+    # the order should be first request and then rate(token)
+    @request_limited
     @rate_limited
     async def acompletion_iter(
         self, messages: list[Message], **kwargs
     ) -> AsyncIterable[LLMResult]:
-        if kwargs.get("include_reasoning"):
-            raise NotImplementedError(
-                "Reasoning with OpenRouter via `include_reasoning` is not supported in streaming mode."
-                "https://github.com/BerriAI/litellm/issues/8631"
-                "Consider `model=deepseek/deepseek-r1` instead."
-            )
         # cast is necessary for LiteLLM typing bug: https://github.com/BerriAI/litellm/issues/7641
         prompts = cast(
-            list[litellm.types.llms.openai.AllMessageValues],
+            "list[litellm.types.llms.openai.AllMessageValues]",
             [m.model_dump(by_alias=True) for m in messages if m.content],
         )
         stream_options = {
             "include_usage": True,
         }
+        # NOTE: Specifically requesting reasoning for deepseek-r1 models
         if kwargs.get("include_reasoning"):
             stream_options["include_reasoning"] = True
 
@@ -663,24 +808,28 @@ class LiteLLMModel(LLMModel):
         logprobs = []
         role = None
         reasoning_content = []
+        used_model = None
         async for completion in stream_completions:
+            if not used_model:
+                used_model = completion.model or self.name
             choice = completion.choices[0]
             delta = choice.delta
-            if hasattr(choice.logprobs, "content"):
-                logprobs.append(choice.logprobs.content[0].logprob or 0)
+            # logprobs can be None, or missing a content attribute,
+            # or a ChoiceLogprobs object with a NoneType/empty content attribute
+            if logprob_content := getattr(choice.logprobs, "content", None):
+                logprobs.append(logprob_content[0].logprob or 0)
             outputs.append(delta.content or "")
             role = delta.role or role
             if hasattr(delta, "reasoning_content"):
-                # NOTE: DeepSeek's reasoning was added in:
-                # https://github.com/BerriAI/litellm/issues/7877
                 reasoning_content.append(delta.reasoning_content or "")
         text = "".join(outputs)
         result = LLMResult(
-            model=self.name,
+            model=used_model,
             text=text,
             prompt=messages,
             messages=[Message(role=role, content=text)],
             logprob=sum_logprobs(logprobs),
+            top_logprobs=extract_top_logprobs(completion),
             reasoning_content="".join(reasoning_content),
         )
 
