@@ -2,7 +2,7 @@ import asyncio
 import logging
 import os
 from collections.abc import Collection
-from typing import ClassVar, Literal
+from typing import ClassVar, Literal, TypeAlias
 from urllib.parse import urlparse
 
 import aiohttp
@@ -31,7 +31,7 @@ CROSSREF_BASE_URL = f"https://{CROSSREF_HOST}"
 GLOBAL_RATE_LIMITER_TIMEOUT = float(os.environ.get("RATE_LIMITER_TIMEOUT", "60"))
 
 MATCH_ALL = None
-MatchAllInputs = Literal[None]
+MatchAllInputs: TypeAlias = Literal[None]  # noqa: PYI061
 MATCH_MACHINE_ID = "<machine_id>"
 
 FALLBACK_RATE_LIMIT = RateLimitItemPerSecond(3, 1)
@@ -47,11 +47,26 @@ TOKEN_FALLBACK_RATE_LIMIT = RateLimitItemPerMinute(30_000, 1)
 # the dynamic IP of the machine will be used to limit the rate of requests, otherwise the
 # user input machine_id will be used.
 
+# Default rate limits for various LLM providers
+OPENAI_DEFAULT_RPM = RateLimitItemPerMinute(  # OpenAI default RPM for most models
+    500, 1
+)
+ANTHROPIC_DEFAULT_RPM = RateLimitItemPerMinute(50, 1)  # Anthropic Claude default RPM
+GOOGLE_DEFAULT_RPM = RateLimitItemPerMinute(15, 1)  # Google Gemini default RPM
+
 RATE_CONFIG: dict[tuple[str, str | MatchAllInputs], RateLimitItem] = {
-    ("get", CROSSREF_BASE_URL): RateLimitItemPerSecond(30, 1),
-    ("get", SEMANTIC_SCHOLAR_BASE_URL): RateLimitItemPerSecond(15, 1),
+    ("get", CROSSREF_HOST): RateLimitItemPerSecond(30, 1),
+    ("get", SEMANTIC_SCHOLAR_HOST): RateLimitItemPerSecond(15, 1),
+    ("client|request", "gpt-3.5-turbo"): RateLimitItemPerMinute(3500, 1),
+    ("client|request", "gpt-4o"): OPENAI_DEFAULT_RPM,
+    ("client|request", "gpt-4"): OPENAI_DEFAULT_RPM,
+    ("client|request", "claude-3"): ANTHROPIC_DEFAULT_RPM,
+    ("client|request", "claude-3.5-sonnet"): ANTHROPIC_DEFAULT_RPM,
+    ("client|request", "gemini-2.0-flash"): GOOGLE_DEFAULT_RPM,
     ("client", MATCH_ALL): TOKEN_FALLBACK_RATE_LIMIT,
-    # MATCH_MACHINE_ID is a sentinel for the machine_id passed in by the caller
+    # MATCH_MACHINE_ID is a sentinel for the machine_id passed in by the caller.
+    # In other words, it means the rate limit will be machine-specific,
+    # not global to all machines in a distributed system.
     (f"get|{MATCH_MACHINE_ID}", MATCH_ALL): FALLBACK_RATE_LIMIT,
 }
 
@@ -85,9 +100,23 @@ class GlobalRateLimiter:
             dict[tuple[str, str | MatchAllInputs], RateLimitItem] | None
         ) = None,
         use_in_memory: bool = False,
+        redis_url: str | None = None,
     ):
+        """Initialize a GlobalRateLimiter instance.
+
+        Args:
+            rate_config: Optional dictionary mapping (namespace, primary_key) tuples to
+                RateLimitItem instances. If not provided, the default RATE_CONFIG is used.
+            use_in_memory: If True, use in-memory storage instead of Redis, even if
+                Redis URL is available.
+            redis_url: Optional Redis URL to use for storage. If not provided, the
+                REDIS_URL environment variable will be used. This parameter allows
+                direct specification of the Redis URL without relying on environment
+                variables.
+        """
         self.rate_config = RATE_CONFIG if rate_config is None else rate_config
         self.use_in_memory = use_in_memory
+        self.redis_url = redis_url
         self._storage: RedisStorage | MemoryStorage | None = None
         self._rate_limiter: MovingWindowRateLimiter | None = None
         self._current_ip: str | None = None
@@ -121,8 +150,9 @@ class GlobalRateLimiter:
     @property
     def storage(self) -> RedisStorage | MemoryStorage:
         if self._storage is None:
-            if os.environ.get("REDIS_URL") and not self.use_in_memory:
-                self._storage = RedisStorage(f"async+redis://{os.environ['REDIS_URL']}")
+            redis_url = self.redis_url or os.environ.get("REDIS_URL")
+            if redis_url and not self.use_in_memory:
+                self._storage = RedisStorage(f"async+redis://{redis_url}")
                 logger.info("Connected to redis instance for rate limiting.")
             else:
                 self._storage = MemoryStorage()
@@ -148,7 +178,7 @@ class GlobalRateLimiter:
         """
         namespace, primary_key = namespace_and_key
 
-        if namespace.startswith("get") and primary_key is not None:
+        if namespace.startswith("get"):
             # for URLs to be parsed correctly, they need a protocol
             if not primary_key.startswith(("http://", "https://")):
                 primary_key = "https://" + primary_key
@@ -172,7 +202,6 @@ class GlobalRateLimiter:
         This parsing logic finds the correct rate limits for a namespace/primary_key.
         It's a bit complex due to the <MATCH ALL> and <MATCH MACHINE ID> placeholders.
         These allow users to match
-
         """
         # the namespace may have a machine_id in it -- we replace if that's the case
         namespace_w_stub_machine_id = namespace
@@ -180,7 +209,7 @@ class GlobalRateLimiter:
 
         # strip off the machine_id, and replace it with the MATCH_MACHINE_ID placeholder
         if namespace.startswith("get"):
-            machine_id = namespace.split("|")[-1]
+            machine_id = namespace.rsplit("|", maxsplit=1)[-1]
             if machine_id != "get":
                 namespace_w_stub_machine_id = namespace.replace(
                     machine_id, MATCH_MACHINE_ID, 1
@@ -240,12 +269,23 @@ class GlobalRateLimiter:
         self, cursor_scan_count: int = 100
     ) -> list[tuple[RateLimitItem, tuple[str, str | MatchAllInputs]]]:
         """Returns a list of current RateLimitItems with tuples of namespace and primary key."""
-        host, port = os.environ.get("REDIS_URL", ":").split(":", maxsplit=2)
+        redis_url = self.redis_url or os.environ.get("REDIS_URL", ":")
+        if redis_url is None:
+            raise ValueError("Redis URL is not set correctly.")
+
+        try:
+            host, port = redis_url.split(":", maxsplit=2)
+        except ValueError as exc:
+            raise ValueError(
+                f"Failed to parse host and port from Redis URL {redis_url!r},"
+                " correctly pass at initialization or set env variable REDIS_URL."
+            ) from exc
 
         if not (host and port):
-            raise ValueError(f"Invalid REDIS_URL: {os.environ.get('REDIS_URL')}.")
+            raise ValueError(f"Invalid Redis URL: {redis_url}.")
 
-        if not isinstance(self.storage, RedisStorage):
+        storage = self.storage
+        if not isinstance(storage, RedisStorage):
             raise NotImplementedError(
                 "get_rate_limit_keys only works with RedisStorage."
             )
@@ -258,7 +298,7 @@ class GlobalRateLimiter:
             while cursor:
                 cursor, keys = await client.scan(
                     int(cursor),
-                    match=f"{self.storage.PREFIX}*",
+                    match=f"{storage.bridge.key_prefix}*",
                     count=cursor_scan_count,
                 )
                 matching_keys.extend(list(keys))
@@ -280,7 +320,8 @@ class GlobalRateLimiter:
     async def get_limit_keys(
         self,
     ) -> list[tuple[RateLimitItem, tuple[str, str | MatchAllInputs]]]:
-        if os.environ.get("REDIS_URL") and not self.use_in_memory:
+        redis_url = self.redis_url or os.environ.get("REDIS_URL")
+        if redis_url and not self.use_in_memory:
             return await self.get_rate_limit_keys()
         return self.get_in_memory_limit_keys()
 
@@ -314,29 +355,29 @@ class GlobalRateLimiter:
         """Returns when the limit is satisfied for the namespace_and_key.
 
         Args:
-            namespace_and_key (:obj:`tuple[str, str]`): is
-                composed of a tuple with namespace (e.g. "get") and a primary-key
-                (e.g. "arxiv.org"). namespaces can be nested with multiple '|',
-                primary-keys in the "get" namespace will be stripped to the domain.
-            rate_limit (:obj:`RateLimitItem | str | None`, optional): Optional
-                RateLimitItem to be used for the namespace and primary-key.
+            namespace_and_key: A two-tuple of namespace (e.g. "get") and a primary key
+                (e.g. "arxiv.org"). Namespaces can be nested with multiple '|'.
+                Primary keys in the "get" namespace will be stripped to the domain.
+            rate_limit: Optional rate limit to be used for the namespace and primary-key.
                 If not provided, RATE_CONFIG will be used to find the rate limit.
                 Can also use a string of the form:
                 [count] [per|/] [n (optional)] [second|minute|hour|day|month|year]
-            machine_id (:obj:`int`, optional): will be used to modify the namespace
-                of GET requests if the primary key is not in the
-                NO_MACHINE_ID_EXTENSIONS list. In that case, the outbound IP will be
-                used to modify the namespace.
-            acquire_timeout (:obj:`float`, optional): is the maximum time (in seconds) to
-                wait for the rate limit to be satisfied.
-            weight (:obj:`int`, optional): is the cost of the request,
-                default is 1. (could be tokens for example)
-            raise_impossible_limits (:obj:`bool`, optional): flag will raise a
-                ValueError for weights that exceed the rate.
+            machine_id: Identifier used only for namespaces that start with "get".
+                - If the host (taken from the primary key, see above namespace_and_key)
+                  matches an entry in `NO_MACHINE_ID_EXTENSIONS`, this value is discarded
+                  in favor of the machine's outbound IP (e.g. "get|198.51.100.4").
+                - If the default value of 0 is used, all requests will be grouped (e.g. "get|0")
+                  into the default shared rate limit bucket.
+                - Otherwise, non-zero values will isolate buckets per machine.
+            acquire_timeout: Maximum wait time (seconds) for the rate limit to be satisfied.
+            weight: Per-unit cost of the request (e.g. cost/token).
+            raise_impossible_limits: Opt-in flag to raise a ValueError for weights
+                that exceed the rate limit.
 
         Raises:
             TimeoutError: if the acquire_timeout is exceeded.
-            ValueError: if the weight exceeds the rate limit and raise_impossible_limits is True.
+            ValueError: if the weight exceeds the rate limit.
+                Only raised if raise_impossible_limits was specified.
         """
         namespace, primary_key = await self.parse_namespace_and_primary_key(
             namespace_and_key, machine_id=machine_id
