@@ -5,17 +5,21 @@ import logging
 import os
 from collections.abc import Callable, Hashable, Iterable
 from contextlib import suppress
-from typing import Any, ClassVar, Self, cast
+from typing import TYPE_CHECKING, Any, ClassVar, Self, cast
 from uuid import UUID
 
 import aiofiles
-import networkx as nx
 from aviary.core import Message, ToolRequestMessage, ToolResponseMessage, join
 from pydantic import BaseModel, ConfigDict, Field, JsonValue, field_validator
 
 from ldp.graph import OpResult
+from ldp.graph.op_utils import _lazy_import_networkx
 from ldp.utils import discounted_returns
 
+if TYPE_CHECKING:
+    from aviary.core import Environment
+
+
 logger = logging.getLogger(__name__)
 
 
@@ -102,6 +106,13 @@ class Trajectory(BaseModel):
 
     traj_id: str | None = None
     steps: list[Transition] = Field(default_factory=list)
+    metadata: dict[str, JsonValue] = Field(
+        default_factory=dict,
+        description=(
+            "Optional JSON metadata on the trajectory, for example the ID of a task"
+            " this trajectory was run on."
+        ),
+    )
 
     @property
     def failed(self) -> bool:
@@ -119,9 +130,17 @@ class Trajectory(BaseModel):
             for s in self.steps:
                 await f.write(s.model_dump_json() + "\n")
 
+    @classmethod
+    async def from_env(cls, env: Environment, **kwargs) -> Self:
+        """Create a Trajectory while propagating environment ID."""
+        traj_metadata = kwargs.pop("metadata", {})
+        with suppress(NotImplementedError, ValueError):
+            traj_metadata["env_id"] = await env.get_id()
+        return cls(metadata=traj_metadata, **kwargs)
+
     @classmethod
     def from_jsonl(cls, filename: str | os.PathLike) -> Self:
-        with open(filename) as f:
+        with open(filename, encoding="utf-8") as f:
             reader = iter(f)
             traj = cls(traj_id=json.loads(next(reader)))
             for json_line in reader:
@@ -162,6 +181,7 @@ class TransitionTree:
         """
         self.root_id = str(root_id)
 
+        nx = _lazy_import_networkx()
         self.tree = nx.DiGraph()  # the actual tree
         self.rev_tree = nx.DiGraph()  # the same as self.tree, but with reversed edges
 
@@ -181,10 +201,10 @@ class TransitionTree:
         if step_id == self.root_id:
             raise ValueError("Root node has no transition.")
 
-        return cast(Transition, self.tree.nodes[step_id]["transition"])
+        return cast("Transition", self.tree.nodes[step_id]["transition"])
 
     def get_weight(self, step_id: str) -> float:
-        return cast(float, self.tree.nodes[step_id]["weight"])
+        return cast("float", self.tree.nodes[step_id]["weight"])
 
     def add_transition(
         self, step_id: str, step: Transition, weight: float = 1.0
@@ -274,7 +294,7 @@ class TransitionTree:
             discount_factor: The discount factor to use when computing cumulative
                 future rewards.
         """
-        for step_id in nx.topological_sort(self.rev_tree):
+        for step_id in _lazy_import_networkx().topological_sort(self.rev_tree):
             step: Transition | None = self.tree.nodes[step_id]["transition"]
             if step is None:
                 continue
@@ -306,7 +326,9 @@ class TransitionTree:
         """
         state_values: dict[str, float] = {}
 
-        for step_id in cast(Iterable[str], nx.topological_sort(self.tree)):
+        for step_id in cast(
+            "Iterable[str]", _lazy_import_networkx().topological_sort(self.tree)
+        ):
             # topological sort means we will update a parent node in-place before
             # descending to its children
 
@@ -347,7 +369,7 @@ class TransitionTree:
             if not trajectory.done:
                 continue
 
-            traj_id_parts = cast(str, trajectory.traj_id).split(":")
+            traj_id_parts = cast("str", trajectory.traj_id).split(":")
 
             for step in trajectory.steps:
                 step_id = ":".join(traj_id_parts[: step.timestep + 2])
@@ -394,7 +416,7 @@ class TransitionTree:
         # old step ID -> new step ID
         node_remap: dict[str, str] = {self.root_id: self.root_id}
 
-        for step_id in nx.topological_sort(self.tree):
+        for step_id in _lazy_import_networkx().topological_sort(self.tree):
             step: Transition | None = self.tree.nodes[step_id]["transition"]
             if step is None:
                 continue
